

function xyw=demo_polygauss_2013(N,polygon_type,function_type)

%--------------------------------------------------------------------------
% REFERENCE PAPER.
%------------------
%
% [1] A. SOMMARIVA and M. VIANELLO 
%     "Gauss-like and triangulation-free cubature over polygons".
%
% THIS DEMO SHOWS HOW TO USE THE PROCEDURE "polygauss" FOR COMPUTING CUBATU-
% RE OVER POLYGONS OF CONTINUOUS FUNCTIONS.
%
%--------------------------------------------------------------------------
% INPUTS.
%---------
%
% N: POINTS USED BY A RULE, I.E. A ONE DIMENSIONAL GAUSSIAN RULE, TO BUILD 
%    THE QUADRATURE RULE ON THE POLYGON.
%
% polygon_type:
%
% SEVERAL POLYGONS ARE DEFINED IN THE m-FILE "define_polygon.m". 
%
% [polygon_type=1]: UNIT SQUARE [0,1]^2.
% [polygon_type=2]: A 6 SIDES CONVEX POLYGON USED IN PAPER [1].
% [polygon_type=3]: A 9 SIDES NONCONVEX POLYGON USED IN THE PAPER [1].
%
% TO DEFINE YOUR OWN POLYGON, PLEASE OPEN THE m-FILE "define_polygon.m" 
% AND ADD IT TO THE LIST. OBSERVE THAT THE FIRST VERTEX IS NOT REPEATED. 
% FOR EXAMPLE, THE UNIT SQUARE IS DEFINED AS "[0 0; 1 0; 1 1; 0 1]" 
% AND NOT "[0 0; 1 0; 1 1; 0 1; 0 0]".
%
%
% function_type:
%
% SEVERAL FUNCTIONS ARE DEFINED IN THE m-FILE "fct2D.m". MORE PRECISELY
%
% [function_type=1]: z=franke(x,y);
% [function_type=2]: z=( (x-0.5).^2 +(y-0.5).^2 ).^(1/2);
% [function_type=3]: z=(x+y).^19;
% [function_type=4]: z=exp(loc_arg); loc_arg=(x-0.5).^2+(y-0.5).^2; 
% [function_type=5]: z=exp(log_arg); loc_arg=-100*((x-0.5).^2+(y-0.5).^2); 
% [function_type=6]: z=cos(30*(x+y));
% [function_type=7]: z=ones(size(x));
%
% TO DEFINE YOUR OWN FUNCTION, PLEASE OPEN THE m-FILE "fct2D.m" AND ADD IT 
% TO THE LIST. 
%
%
%--------------------------------------------------------------------------
% OUTPUTS.
%----------
%
% xyw     : THE GAUSS LIKE FORMULA PRODUCES THE NODES (xyw(:,1),xyw(:,2)) 
%           AND THE WEIGHTS xyw(:,3) OF A CUBATURE RULE ON THE POLYGON.
%
%--------------------------------------------------------------------------
% ATTACHED FUNCTIONS.
%---------------------
% 1. exact_integrals.m
% 2. f.m
% 3. define_polygon.m
%
%--------------------------------------------------------------------------
% EXAMPLE.
%---------------------
% >> xyw=demo_polygauss_2013(20,2,1);
% 
%  	 [POLYGON]: CONVEX POLYGON
%  	 ---------------------------------------------------------
%  	 [SIDES   ]:  6 [DEGREE ]:    20 [PTS]:  2520
%  	 [DOMAIN  ]:  2 [FUNCTION     ]:  1 [CPUTIME]: 1.00e-02
%  	 [PGAUSS RESULT ]: 0.260067709012239
%  	 [EXACT RESULT  ]: 0.260067709012133
%  	 [ABSOLUTE ERROR]: 1.061928323053962e-13
%  	 [RELATIVE ERROR]: 4.083276340179625e-13
%  	 ---------------------------------------------------------
% >>
%--------------------------------------------------------------------------
%% Copyright (C) 2007-2013 Marco Vianello and Alvise Sommariva
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

%% Authors:  
%% Marco Vianello    <marcov@euler.math.unipd.it>
%% Alvise Sommariva  <alvise@euler.math.unipd.it>
%% Date: March 7, 2007
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% SETTING DEFAULT.
%-------------------
% [plot_polygon = 0] NO PLOTS.
% [plot_polygon = 1] IT PLOTS THE DOMAIN AND THE CUBATURE NODES.                   
%--------------------------------------------------------------------------
plot_polygon = 1;

if nargin < 3
    function_type=1;
end

if nargin < 2
    polygon_type=1;
end

if nargin < 1
    N=10;
end


%--------------------------------------------------------------------------
%                    THE MAIN PROGRAM STARTS HERE.
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% DEFINE POLYGON.
%--------------------------------------------------------------------------
% "define_polygon.m" DEFINES THE COORDINATES OF THE VERTICES (REPEATING AT
% THE END THE FIRST POINT). THE VARIABLE "polygon_type" IS DEFINED ABOVE. 
% "polygon_sides" IS A VECTOR HAVING AS COMPONENTS THE VERTICES OF THE 
% COMPONENT, DESCRIBED COUNTERCLOCKWISE. 
%--------------------------------------------------------------------------
polygon_sides=define_polygon(polygon_type);

%--------------------------------------------------------------------------
% COMPUTING INTEGRAL 
%--------------------------------------------------------------------------
% "cubature_result", NODES "(x_nodes,y_nodes)" AND WEIGHTS "weights".
% "fct2D.m" IS AN m-file DEFINED BY THE USER, HAVING "function_type" AS
% VARIABLE THAT SWITCHES BETWEEN FUNCTIONS. SEE THE FILE "fct2D.m" FOR 
% DETAILS.
%--------------------------------------------------------------------------
cputime1=cputime;
xyw=polygauss_2013(N,polygon_sides);
cputime2=cputime;
x_nodes=xyw(:,1);
y_nodes=xyw(:,2);
weights=xyw(:,3);

f_xy = f(x_nodes,y_nodes,function_type);
cubature_result=weights'*f_xy;

%--------------------------------------------------------------------------
% SOME STATS.
%--------------------------------------------------------------------------
fprintf('\n \t [SIDES   ]: %2.0f [DEGREE ]: %5.0f [PTS]: %5.0f',...
    length(polygon_sides(:,1))-1, N,size(x_nodes,1)*size(x_nodes,2));
fprintf('\n \t [DOMAIN  ]: %2.0f [FUNCTION     ]: %2.0f [CPUTIME]: %2.2e',...
    polygon_type,function_type,cputime2-cputime1);

fprintf('\n \t [PGAUSS RESULT ]: %5.15f',cubature_result); 
exact_result=exact_integrals(polygon_type,function_type);
fprintf('\n \t [EXACT RESULT  ]: %5.15f',exact_result); 
fprintf('\n \t [ABSOLUTE ERROR]: %5.15e',abs(cubature_result-exact_result)); 
if abs(exact_result) > 0
    relerr=abs(cubature_result-exact_result)/abs(exact_result);
    fprintf('\n \t [RELATIVE ERROR]: %5.15e',relerr); 
end
fprintf('\n \t ---------------------------------------------------------');

%--------------------------------------------------------------------------
% PLOT POLYGONS.
%--------------------------------------------------------------------------
if plot_polygon == 1
    clf;
    hold on; 
    fill(polygon_sides(:,1),polygon_sides(:,2),'w'); 
    plot(x_nodes,y_nodes,'k.'); 
    hold off; 
end
 
fprintf('\n');






function polygon_sides=define_polygon(polygon_type)

%-------------------------------------------------------------------------------
% POLYGON DEFINITION. 
%-------------------------------------------------------------------------------
%
% INPUT:
%
% [polygon_type]: PARAMETER THAT CHOOSES THE POLYGON. 
%
% OUTPUT:
% [polygon_sides]: DEFINES THE COORDINATES OF THE VERTICES (REPEATING AT THE END 
%                  THE FIRST POINT). THE VARIABLE "polygon_type" IS DEFINED ABOVE;
%                  "boundary_pts" IS A VECTOR HAVING AS COMPONENTS THE
%                  VERTICES OF THE COMPONENT, DESCRIBED COUNTERCLOCKWISE. 
%                  
%-------------------------------------------------------------------------------
% OBSERVE THAT THE FIRST VERTEX IS NOT REPEATED. 
% FOR EXAMPLE, THE UNIT SQUARE IS DEFINED AS "[0 0; 1 0; 1 1; 0 1]" 
% AND NOT "[0 0; 1 0; 1 1; 0 1; 0 0]".
%-------------------------------------------------------------------------------

switch polygon_type
    
case 1
    fprintf('\n \t [POLYGON]: UNIT SQUARE [0,1]^2'); 
    polygon_sides=[0 0; 1 0; 1 1; 0 1]; 
case 2
    fprintf('\n \t [POLYGON]: CONVEX POLYGON'); 
    polygon_sides=[0.1 0; 0.7 0.2; 1 0.5; 0.75 0.85; 0.5 1; 0 0.25];
case 3    
    fprintf('\n \t [POLYGON]: NON CONVEX POLYGON'); 
    polygon_sides=(1/4)*[1 0; 3 2; 3 0; 4 2; 3 3; 3 0.85*4; 2 4; 0 3; 1 2];
end

%-------------------------------------------------------------------------------
% "CLOSE" THE POLYGON (IT IS NEEDED BY INPOLYGON AND BY DBLQUAD (FOR TESTS)).
%-------------------------------------------------------------------------------
polygon_sides=[polygon_sides; polygon_sides(1,:)];











function exact_result=exact_integrals(polygon_type,function_type)

% ALL THESE INTEGRALS HAS BEEN PRECOMPUTED BY HIGH DEGREE GAUSS LEGENDRE CUBATURE
% OVER POLYGONS.

switch polygon_type
    
case 1
    switch function_type
    case 1
        exact_result=0.406969589491556; 
    case 2
        exact_result=0.382597858232106; 
    case 3
        exact_result=4993.214285714256200; 
    case 4
        exact_result=1.188043774905800; 
    case 5
        exact_result=0.031415926535801; 
    case 6
        exact_result=0.000289906533545;
    case 7
        exact_result=1.000000000000000;
    end
    
case 2
    switch function_type
    case 1
        exact_result=0.260067709012133;
    case 2
        exact_result=0.156825125547501; % 10 DIGITS PRECISION.
    case 3
        exact_result=169.704343403127520; % 12 DIGITS PRECISION.
    case 4
        exact_result=0.593459365720563;
    case 5
        exact_result=0.031414528632393;
    case 6
        exact_result=0.008421180941490;
    case 7
        exact_result=0.535000000000000;
    end
    
case 3
    switch function_type
    case 1
        exact_result=0.175565704080668;
    case 2
        exact_result=0.139381456868220;   % 12 DIGITS ARE CORRECT.
    case 3
        exact_result=130.841234986796820; % 13 DIGITS ARE CORRECT.
    case 4
        exact_result=0.531841554503018;
    case 5
        exact_result=0.031220838971539;
    case 6
        exact_result=0.014222050981512;
    case 7
        exact_result=0.481250000000000;
    end
    
end







function z=f(x,y,function_type)

%-------------------------------------------------------------------------------
% FUNCTION DEFINITION. 
%-------------------------------------------------------------------------------
%
% INPUT:
%
% [x,y]: EVALUATE THE FUNCTION IN THE POINTS (x,y). "x", "y" ARE COLUMN VECTORS.
%
% [function_type]: PARAMETER THAT CHOOSES THE FUNCTION.
%
% OUTPUT:
%
% [z]: VALUE OF THE FUNCTION IN THE POINTS (x,y). IT IS A COLUMN VECTOR.
%
%-------------------------------------------------------------------------------

switch function_type
case 1
    z=franke(x,y);
case 2
    z=( (x-0.5).^2 +(y-0.5).^2 ).^(1/2);
case 3
    z=(x+y).^19;
case 4
    s=(x-0.5).^2+(y-0.5).^2;
    z=exp(s);
case 5
    s=(x-0.5).^2+(y-0.5).^2; s=-100*s;
    z=exp(s);
case 6
    z=cos(30*(x+y));
case 7
    z=ones(size(x));
end


